
export default class NwfValidator {

    myEl: HTMLInputElement;
    invalidMessage: string;
    emptyMessage: string;

    static $inject = ['$element'];

    constructor($element) {
        this.myEl = $element.get(0);
        let elType = this.myEl.type;
        // Checkboxes don't need invalid handler
        if (elType !== 'checkbox') {
            this.myEl.addEventListener('invalid', this.invalidHandler.bind(this));
        } else {
            this.myEl.addEventListener('change', this.resetCustomValidity.bind(this));
        }
        this.myEl.addEventListener('input', this.resetCustomValidity.bind(this));
    }

    $onInit() {
        // Checkboxes need their values set
        if (this.myEl.type == 'checkbox') {
            this.myEl.setCustomValidity(this.emptyMessage);
        }
    }

    invalidHandler() {

        if (this.myEl.validity.valueMissing) {
            this.myEl.setCustomValidity(this.emptyMessage);
        } else if (!this.myEl.validity.valid) {
            this.myEl.setCustomValidity(this.invalidMessage);
        } else {
            this.myEl.setCustomValidity('');
        }

    }

    resetCustomValidity() {
        // First I reset
        this.myEl.setCustomValidity('');
        // Then I call my invalidHandler
        this.invalidHandler();
    }

}
