import * as ng from 'angular';
import * as _ from 'lodash';

export default class ColumnChooserController {
    private option;
    public options;
    private _options;

    public gridApi;

    public selections;
    private _selections;

    private changesInFlight = false;

    private _selectionsBeforeChange;
    private _optionsBeforeChange;

    private myStorageKey;

    private popoverPlacement = 'auto bottom-right';

    private isPopoverOpen = false;
    private schemaOptions = []; //Holds the original schema handed in, as a reference of the desired order.

    static $inject = ['$scope', '$timeout', '$state'];

    constructor(private $scope: angular.IScope, private $timeout, private $state) {
        //I generate a key for this state and controller class name as a unique key. Of course, this works when there is only one of this kind but it's ok FTM
        var stateName = this.$state.current.name;
        var myWidget = this.constructor['name'];
        this.myStorageKey = stateName + '.' + myWidget;

        var myStickySels = JSON.parse(window.sessionStorage.getItem(this.myStorageKey));

        $scope.$watchGroup((['columnChooserCtrl.options', 'columnChooserCtrl.selections']), (newVal, oldVal) => {
            if (!newVal[0] && !oldVal[0]) {
                return;
            }

            var myInitialOptions = newVal[0];
            this._options = this.cloneMyObj(myInitialOptions);

            /* If I have stickySelections in local storage, I want to prune down my set from the server just to those
                I just persist an array of field names to be lean, so I need to reconstitute the real columns from the entire set.
             */
            var allMyAvailableSelectionsFromServer = newVal[1];
            var stickySelections = _.map(myStickySels, function (i: any) { return _.find(allMyAvailableSelectionsFromServer, { field: i }); });
            if (stickySelections.length) {
                this._selections = stickySelections;
            } else {
                // The default is all selections handed in from the controller scope
                this._selections = this.cloneMyObj(allMyAvailableSelectionsFromServer);
            }
            //set the selections on the grid, we need a copy so we aren't bound (we want only "apply()" to do the work
            this.gridApi.columnDefs = this.cloneMyObj(this._selections);
            //Select the first option by default
            this.option = this._options[0];
            this.schemaOptions = this.cloneMyObj(this._options);
            this.refresh();
        });


        // Connecting our outsideClick handler ourselves
        var bodyEl = document.body;
        var onBodyClick = (evt) => {
            /* I check the event's target has any parentage who's class begins with nwf-column-chooser-
                that way I can differentiate any clicks that are not inside my widget, and cause
                the parent popover to come down.
                This selector here is - any element who's class attribute starts with nwf-column-chooser-
             */
            var target = ng.element(evt.target);
            if (target.closest('[class*="nwf-column-chooser-"]').length) {
                return;
            } else {
                this.closePopover();
                $timeout(() => $scope.$digest());
            }
        };

        bodyEl.addEventListener('click', onBodyClick);
        //Removing handler upon destruction
        $scope.$on('$destroy', () => {
            bodyEl.removeEventListener('click', onBodyClick);
        });
    }

    refresh() {
        // find the selected from the options
        var initialIdsToSpliceFrom = [];
        _.forEach(this._selections, (selectionToRemoveFromOptions) => {
            if (selectionToRemoveFromOptions && selectionToRemoveFromOptions.field) {
                var selIndex = _.findIndex(this._options, { field: selectionToRemoveFromOptions.field });
                var cloneOfSelection = this.cloneMyObj(selectionToRemoveFromOptions);
                initialIdsToSpliceFrom.push(selectionToRemoveFromOptions.field);
            }
        });
        // now splice them out of the options
        _.forEach(initialIdsToSpliceFrom, (index) => {
            var selIndex = _.findIndex(this._options, { field: index });
            this._options.splice(selIndex, 1);
        });
    }

    closePopover() {
        this.isPopoverOpen = false;
    }

    togglePopover(event) {
        //I take a snapshot when we open for use when I cancel()
        // logic is, if I am opening (from closed) and I don't have changes in Flight, I snapshot it.
        if (!this.isPopoverOpen && this.changesInFlight == false) {
            this._selectionsBeforeChange = this.cloneMyObj(this._selections);
            this._optionsBeforeChange = this.cloneMyObj(this._options);
        }
        this.isPopoverOpen = !this.isPopoverOpen;
    }

    moveSelection(fromIndex: number, toIndex: number) {
        var sels = this._selections;
        var element = sels[fromIndex];
        if (fromIndex === 0 && toIndex === -1) {
            return;
        }
        sels.splice(fromIndex, 1);
        sels.splice(toIndex, 0, element);
        // flip my bit on changesInFlight to let cancel know what to do.
        this.changesInFlight = true;
    }

    selectOption() {
        // Select means to take the column out of options and move to selected
        //If I have no options to select do nothing.
        if (!this._options.length || !this.option) {
            return;
        }
        var selOption = this.option;
        /* remove me from available (options) */
        var opts = this._options;
        var selIndex = _.findIndex(opts, { field: selOption.field });
        opts.splice(selIndex, 1);

        /* put me in columns to display (selections) */
        var insertIndex = this.findInsertIndex();
        this._selections.splice(insertIndex, 0, selOption);

        //preselect the one below for usability
        this.option = opts[selIndex] ? opts[selIndex] : opts[selIndex - 1];
        // flip my bit on changesInFlight to let cancel know what to do.
        this.changesInFlight = true;
    }

    deselectOption(option) {
        //Deselect means I am taking a selected column and moving it to available (the X)
        /* Find my self in the schema options so I Can be put back where I belong */
        var originalIndex = _.findIndex(this.schemaOptions, { field: option.field });
        /* put myself back in options where I belong at the index I was born in (restoring order) */
        this._options[originalIndex] = option;
        this.option = option;
        /* Now put slice myself from selections */
        var selIndex = _.findIndex(this._selections, { field: option.field });
        this._selections.splice(selIndex, 1);
        // flip my bit on changesInFlight to let cancel know what to do.
        this.changesInFlight = true;
    }

    moveSelectionUp(index: number) {
        this.moveSelection(index, index - 1);
    }

    moveSelectionDown(index: number) {
        this.moveSelection(index, index + 1);
    }

    cloneMyObj(obj) {
        return JSON.parse(JSON.stringify(obj));
    }

    resetColumns() {
        //Set my selections back to the original set of options when I was born (from the server)
        this._selections = this.cloneMyObj(this.options);
        this.refresh();
        this.gridApi.columnDefs = this.cloneMyObj(this._selections);
        window.sessionStorage.removeItem(this.myStorageKey);
        this.closePopover();
        this.changesInFlight = false;
    }

    cancel() {
        // if I have made any changes, I reset them to the snapshotted copy.
        if (this.changesInFlight) {
            this._selections = this._selectionsBeforeChange;
            this._options = this._optionsBeforeChange;
        }
        this.closePopover();
        this.changesInFlight = false;
    }

    apply() {
        this.gridApi.columnDefs = this.cloneMyObj(this._selections);
        var selectedFields = _.map(this._selections, 'field');
        window.sessionStorage.setItem(this.myStorageKey, JSON.stringify(selectedFields));
        this.closePopover();
        this.changesInFlight = false;
    }

    findInsertIndex() {
        let i = 0;
        let j = 0;
        let index;

        if (this._selections.length === 0) {
            // no column is selected, insert at 0
            index = i;
        } else {
            // If the current schemaOption match option, then it means the current 
            // selection is after the option (and previous selection is before 
            // option), we should insert option into _selections at the index of 
            // current selection
            while (this.schemaOptions[j].field !== this.option.field) {
                if (this._selections[i].field === this.schemaOptions[j].field) {
                    // The current selection is before option, moved to search 
                    // next selection
                    i++;
                }
                // the current selection not match the current schemaOption, compare 
                // next schemaOption
                j++;
            }
            index = i;
        }

        return index;
    }
}
