import * as loginFailedModalTemplate from './nwfLogin-failed-dialog.html';
import * as loginForgotPasswordTemplate from './nwfLogin-forgot-password-info.html';
import * as loginResetPasswordTemplate from './nwfLogin-reset-password-info.html';
import * as loginResetPasswordResultTemplate from './nwfResetPasswordResult-modal.html';

interface stateParams {
    user: string;
    authToken: string;
}

export default class NwfLoginController {

    private isOemVendor = false;
    private supportHref;
    private homeVendorHref
    private homeHref;

    titleElement = null;

    username = null;
    password = null;


    //Forgot Password
    forgotPassword = false;
    newPassword;
    confirmPassword;
    confirmNewPassword;

    message;
    success;


    //Reset Password
    resetPassword = false;
    authToken = null;
    resetPasswordMatch = true;

    errorCode;
    inFlight;

    loginFailedController;
    resetPasswordSuccessController;

    static $inject = ['$uibModal', '$rootScope', '$element', 'authStrategy', '$i18next'];

    constructor(private $uibModal, private $rootScope, private $element, private authStrategy, private $i18next) {

        var errorCode;
        // Controller used by login failed dialog.
        this.loginFailedController = ['$scope', '$uibModalInstance',
            function ($scope, $uibModalInstance) {
                $scope.errorCode = errorCode;
                $scope.ok = function () {
                    $uibModalInstance.close();
                };
            }
        ];

        // Controller used by reset password modal instance on success. The user is redirected to login page.
        this.resetPasswordSuccessController = ['$scope', '$uibModalInstance',
            function ($scope, $uibModalInstance) {
                $scope.ok = function () {
                    $uibModalInstance.close();
                    window.location.href = '/';
                };
            }
        ];

    }

    /* onLoginForm Submit */
    submitFn() {

        this.inFlight = true;

        var credentials = {
            username: this.username,
            password: this.password
        };

        return this.authStrategy.authenticateFn(credentials).then(() => {
            this.inFlight = false;
        }).catch((error) => {
            this.errorCode = error.status;
            this.inFlight = false;
            this.$uibModal.open({
                template: loginFailedModalTemplate,
                controller: this.loginFailedController
            });
        });
    }


    //Function to toggle the scope variable.
    forgotPasswordToggleFn() {
        this.forgotPassword = (this.forgotPassword === false) ? true : false;
    }

    //Forgot Password cancel action.
    forgotPasswordCancelFn() {
        this.forgotPassword = false;
    }


    //forgot password on submit.
    forgotPasswordSubmitFn() {

        var credentials = {
            username: this.username
        };

        return this.authStrategy.forgotPasswordAuthenticateFn(credentials).then(() => {
            var forgotPasswordToggle = this.$uibModal.open({
                template: loginForgotPasswordTemplate,
                controller: ['$scope', 'forgotPasswordName', function ($scope, forgotPasswordName) {
                    $scope.username = forgotPasswordName;
                }],
                resolve: {
                    forgotPasswordName: function () {
                        return credentials.username;
                    }
                }
            });

            //Once done do back to the Login form.
            forgotPasswordToggle.result.then(() => {
                this.forgotPasswordToggleFn();
            });
        });
    }

    //Reset password on cancel.
    resetPasswordCancelFn() {
        window.location.href = document.location.origin;
    }

    //Reset password on submit.
    resetPasswordSubmitFn() {
        var data = {
            user: this.username,
            newPassword: this.newPassword,
            confirmNewPassword: this.confirmNewPassword,
            authToken: this.authToken
        };

        //Check if passwords match
        if (this.newPassword !== this.confirmNewPassword) {
            this.resetPasswordMatch = false;
            return;
        } else {
            this.resetPasswordMatch = true;
        }

        return this.authStrategy.resetPasswordFn(data).then(() => {
            this.message = this.$i18next.t("nwfjs:resetPasswordSucceed");
            this.success = true;

            //Show the modal with success message from the server.
            this.$uibModal.open({
                template: loginResetPasswordResultTemplate,
                scope: this,
                controller: this.resetPasswordSuccessController
            });
        }).catch(function (error) {
            this.message = this.$i18next.t("nwfjs:resetPasswordFailed");
            if (error != null) {
                this.message = error.message;
            }
            this.success = false;
            //Show the  modal with failure message from the server.

            this.$uibModal.open({
                template: loginResetPasswordResultTemplate,
                scope: this,
                controller: this.loginFailedController
            });
        });
    }

    $onInit() {

        let netappHome = 'http://netapp.com';
        this.isOemVendor = false;
        this.supportHref = this.supportHref || 'http://mysupport.netapp.com';
        this.homeHref = this.homeHref || netappHome;
        if (this.homeHref !== netappHome) {
            this.isOemVendor = true;
        }

        // If the server sends a username and authToken, the only use case we have is that user has send a request to reset the password with this URL.
        // Set the boolean to reset password to be true.

        if (document.location.hash.indexOf('?') !== -1) {
            var $stateParams = document.location.hash.split('?')[1].split('&');

            var stateParams = <stateParams>{};
            $stateParams.forEach(function (item) {
                var pair = item.split('=');
                stateParams[pair[0]] = pair[1];
            });

            if (stateParams.user !== undefined && stateParams.authToken !== undefined) {
                this.username = stateParams.user;
                this.resetPassword = true;
                this.forgotPassword = false;
                this.authToken = stateParams.authToken;
                this.$rootScope.$broadcast('nwfShowLogin');
            }
        }
    }

    $postLink() {
        var el = this.$element.get(0);
        var frm = <HTMLFormElement>el.querySelector('#nwf-login-form');

        var resetMyForm = function () {
            for (var i = 0; i < frm.elements.length; i++) {
                let h = 'hasBeenBlurred';
                if (frm.elements[i][h]) {
                    delete frm.elements[i][h];
                }
            }
        };

        /* When I hide, I want to reset my form (blowing out the values) and then reset the validation */
        this.$rootScope.$on('nwfHideLogin', function () { frm.reset(); });

        /* 
        This is something I'm not happy with. When I reset, I need to flip the hasBeenBlurred bit so my nwfValidation works as if it has been reset
        it is rare that I have a reset case, and for now I just have this on the login form. 
        */

        frm.addEventListener('reset', resetMyForm);
    }

}
